/*******************************************************************************
 * ALMA - Atacama Large Millimeter Array
 * Copyright (c) ESO - European Southern Observatory, 2011
 * (in the framework of the ALMA collaboration).
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *******************************************************************************/
package alma.obops.tmcdb.alarms.ui.actions.edit;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import alma.acs.tmcdb.ReductionThreshold;
import alma.obops.tmcdb.alarms.ui.editors.ReductionThresholdEditor;
import alma.obops.tmcdb.alarms.ui.editors.inputs.ReductionThresholdEditorInput;
import alma.obops.tmcdb.alarms.ui.perspectives.AlarmsPerspective;
import alma.obops.tmcdb.alarms.ui.tree.helpers.ReductionThresholdHelper;
import alma.obops.tmcdb.alarms.ui.utils.RcpUtils;
import alma.obops.tmcdbgui.domain.IModelChangeListener;

public class EditReductionThresholdAction extends Action implements ISelectionListener, IWorkbenchAction 
{
	private IStructuredSelection _selection;
	private IWorkbenchWindow _window;
	private String ID = "alma.obops.tmcdb.alarms.ui.actions.EditReductionThresholdAction";
	private IModelChangeListener modelChangeListener;

	public EditReductionThresholdAction(IWorkbenchWindow window, IModelChangeListener listener)
	{
		_window = window;
    	setId(ID);
        setText("Edit Reduction Threshold");
		setToolTipText("Opens the Reduction Threshold in an editor");
		setImageDescriptor(RcpUtils.getImageDescriptor("icons/reduction-threshold.png"));
		this.modelChangeListener = listener;
    	_window.getSelectionService().addSelectionListener(this);	
	}
	
	public void run() 
	{
		try {
			ReductionThreshold reductionThreshold = (ReductionThreshold)_selection.getFirstElement();
			try {
				reductionThreshold = ReductionThresholdHelper.findReductionThreshold(reductionThreshold);
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException("Could not hydrate reduction threshold", e);
			}
			ReductionThresholdEditorInput editorInput = new ReductionThresholdEditorInput(reductionThreshold, modelChangeListener);
			_window.getWorkbench().showPerspective(AlarmsPerspective.ID, _window);
			_window.getActivePage().openEditor(editorInput, ReductionThresholdEditor.ID);
		} catch (Exception e) {
			e.printStackTrace();
			RcpUtils.errorMessage(e, this._window.getShell(), "Error encountered", "Problem editing reduction threshold");
		}
	}

	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) 
	{
		if( selection instanceof IStructuredSelection ) 
		{
			_selection = (IStructuredSelection)selection;
			setEnabled( _selection.size() == 1 &&
			            _selection.getFirstElement() instanceof ReductionThreshold);	
		}
	}

	@Override
	public void dispose() {
		_window.getSelectionService().removeSelectionListener(this);
	}
}
