/*******************************************************************************
 * ALMA - Atacama Large Millimeter Array
 * Copyright (c) ESO - European Southern Observatory, 2011
 * (in the framework of the ALMA collaboration).
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *******************************************************************************/
package alma.obops.tmcdbgui.handlers;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;

import alma.acs.tmcdb.Container;
import alma.acs.tmcdb.ContainerStartupOption;
import alma.obops.tmcdb.alarms.ui.utils.RcpUtils;
import alma.obops.tmcdbgui.utils.GuiUtils;
import alma.obops.tmcdbgui.wizards.NewContainerStartupOptionWizard;

public class NewContainerStartupOptionAction extends Action implements
		ISelectionListener, IWorkbenchAction 
{

	private IStructuredSelection _selection;
    private IWorkbenchWindow _window;
    private final static String ID = "new_containerstartupoption.action";
    private Container container;

    public NewContainerStartupOptionAction() {}

    public NewContainerStartupOptionAction(IWorkbenchWindow window) {
    	_window = window;
    	_window.getSelectionService().addSelectionListener(this);
    	setEnabled(GuiUtils.isGodUser());
    	setId(ID);
    	this.setEnabled(GuiUtils.isGodUser());
    	this.setToolTipText("Creates a new ContainerStartupOption");
    	this.setText("New ContainerStartupOption...");
    	this.setImageDescriptor(RcpUtils.getImageDescriptor("icons/containerstartupoption-new.png"));
    }

	@Override
	public void run() {
		NewContainerStartupOptionWizard wizard = new NewContainerStartupOptionWizard(container);
    	wizard.init(_window.getWorkbench(), _selection);
    	WizardDialog dialog = new WizardDialog(_window.getShell(), wizard);
    	dialog.create();
    	dialog.open();
	}

	@Override
	public void dispose() {
		_window.getSelectionService().removeSelectionListener(this);
	}

	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if( selection instanceof IStructuredSelection && GuiUtils.isGodUser()) 
		{
			_selection = (IStructuredSelection)selection;
			if( _selection.getFirstElement() instanceof ContainerStartupOption[]) 
			{
				setEnabled(true);
				container = ((ContainerStartupOption[])_selection.getFirstElement())[0].getContainer();
			}
			else if(_selection.getFirstElement() instanceof Container) {
				setEnabled(true);
				container = (Container)_selection.getFirstElement();
			}
			else {
				setEnabled(false);
			}
		}
		else {
			setEnabled(false);
		}
	}

	public void setContainer(Container cont)
	{
		this.container = cont;
	}
}
