/*******************************************************************************
 * ALMA - Atacama Large Millimeter Array
 * Copyright (c) ESO - European Southern Observatory, 2011
 * (in the framework of the ALMA collaboration).
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *******************************************************************************/
package alma.obops.tmcdbgui.wizards.support;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Text;

/**
 * Class for verifying input on integer fields.
 * @author sharring
 */
public class IntegerStringVerifyListener implements VerifyListener 
{
	private int numDigitsAllowed;
	
	/**
	 * Constructor.
	 * @param numDigitsAllowed the maximum number of digits in the integer
	 */
	public IntegerStringVerifyListener(int numDigitsAllowed)
	{
		this.numDigitsAllowed = numDigitsAllowed;
	}
	
	@Override
	public void verifyText(VerifyEvent e) 
	{
		e.doit = true;

		Text control = (Text)e.widget;
		String text = control.getText();
		if(text != null && text.trim().length() > 0) 
		{
			if(Character.isDigit(e.character) && text.length() < numDigitsAllowed)
			{
				e.doit = true;
			}
			else if(Character.isDigit(e.character) && e.start < e.end) {
				e.doit = true;
			}
			else if(e.keyCode == SWT.BS || e.keyCode == SWT.DEL) {
				e.doit = true;
			}
			else {
				e.doit = false;
			}
		}
	}
}